macroScript BakePFlow2Obj category:"Bobo_s Tools"
(
	----------------------------------------------------------------------------
	--BAKE PARTICLE FLOW PARTICLES TO OBJECTS
	--Version 1.2
	--Code by Borislav "Bobo" Petrov
	--Started: 		2008/02/16
	--Last Modified: 	2009/03/25
	----------------------------------------------------------------------------
	persistent global BakePFlowToObjects_PFlow --this will be the particle flow to bake
	persistent global BakePFlowToObjects_Proxy --this will be the object to distribute
	local theIniFile = getDir #plugcfg + "\\BakePFlowToObjects.ini"	
	
	global BakePFlowToObjects_Rollout
	try(destroyDialog BakePFlowToObjects_Rollout)catch()
	rollout BakePFlowToObjects_Rollout "Bake PFlow To Objects"
	(
		fn filterPF obj = classof obj == PF_Source
		pickbutton pck_pflow ">>Pick a Particle Flow..." width:180 align:#center filter:filterPF
		dropdownlist ddl_bakeMode items:#("Distribute Source Instances","Use Source's Children Instances","Bake PFlow Mesh") width:180 align:#center
		pickbutton pck_source ">>Pick a Distribution Source..." width:180 align:#center 
		dropdownlist ddl_timeMode items:#("Current Time Segment","Current Frame")
		button btn_bakeit "BAKE!" width:180 height:30 align:#center enabled:false
		button btn_deleteBaked "Delete Baked Objects" width:180 align:#center 
		fn updateEnabledState =
		(
			btn_bakeit.enabled = case ddl_bakeMode.selection of
			(
				1: isValidNode BakePFlowToObjects_PFlow and isValidNode BakePFlowToObjects_Proxy
				2: isValidNode BakePFlowToObjects_PFlow and isValidNode BakePFlowToObjects_Proxy and BakePFlowToObjects_Proxy.children.count > 0
				3: isValidNode BakePFlowToObjects_PFlow 
			)	
			pck_pflow.text = if isValidNode BakePFlowToObjects_PFlow then BakePFlowToObjects_PFlow.name else ">>Pick a Particle Flow..."
			pck_source.text = if isValidNode BakePFlowToObjects_Proxy then BakePFlowToObjects_Proxy.name else ">>Pick a Distribution Source..."
		)
		
		on pck_pflow picked obj do
		(
			if isValidNode obj do BakePFlowToObjects_PFlow = obj
			updateEnabledState()
		)
		
		on pck_source picked obj do
		(
			if isValidNode obj do BakePFlowToObjects_Proxy = obj
			updateEnabledState()
		)		
		
		on ddl_bakeMode selected itm do updateEnabledState()
		
		on btn_bakeit pressed do
		(
			delete (execute ("$'BAKE_" + BakePFlowToObjects_PFlow.name + "_instance'*")) --delete old instances from same pflow
			local InstancesArray = #() --init. array to store the instances
			local bornIDStorage = #() --init. array to collect born IDs
			local lastBornIDStorage = #() --init array to collect previous frame's born IDs
			local firstFrame = animationrange.start.frame
			local lastFrame = animationrange.end.frame
			if ddl_timeMode.selection == 2 do firstFrame = lastFrame = sliderTime.frame
			progressStart "Baking Particles..."
			local cnt = 0
			local numFrames = lastFrame - firstFrame 
			for t = firstFrame to lastFrame do --loop through the scene time range
			(
				cnt += 1
				if not (progressUpdate (100.0*cnt/numFrames)) do exit
				slidertime = t --set the scene time to force an update of PFlow
				theCount = BakePFlowToObjects_PFlow.numParticles() --get the particle count 
				lastBornIDStorage = for i in bornIDStorage collect i --copy the previous frame's array
				bornIDStorage = #() --reinit. the current frame's array
				for p = 1 to theCount do --for each particle on the current frame,
				(
					BakePFlowToObjects_PFlow.particleIndex = p --set the index 
					theID = BakePFlowToObjects_PFlow.particleID  --and get the ID
					if theID > 0 do  --if the ID is positive,
					(
						append bornIDStorage theID	--add the ID to the current frame's IDs array		
						if InstancesArray[theID] == undefined do --if no instance has been created yet for this particle,
						(
							case ddl_bakeMode.selection of --depending on the baking mode,
							(
								1: (
										--if the source is a light with an on/off switch,
										theInstance = if superclassof BakePFlowToObjects_Proxy == Light and hasProperty BakePFlowToObjects_Proxy #multiplier then
											copy BakePFlowToObjects_Proxy --copy the proxy object
										else
											instance BakePFlowToObjects_Proxy --instance the proxy object
									)
								2: (
										local theSources = for o in BakePFlowToObjects_Proxy.children collect o
										local theRandomObject = theSources[random 1 theSources.count]
										--if the source is a light with an on/off switch,
										theInstance = if superclassof theRandomObject == Light and hasProperty theRandomObject #multiplier then
											copy theRandomObject --copy the proxy object
										else
											instance theRandomObject --instance the proxy object
									)
								3: (
										theInstance = Editable_Mesh() --create an empty mesh
										theInstance.mesh = BakePFlowToObjects_PFlow.particleShape --and set its TriMesh to the particle's mesh
									)
							)--end case
							theInstance.transform = BakePFlowToObjects_PFlow.particleTM --set the transform of the new object to the particle's TM
							theInstance.name = uniquename ("BAKE_" + BakePFlowToObjects_PFlow.name + "_instance") --rename the instance based on the PFlow's name
							if ddl_timeMode.selection == 1 do
							(
								if ddl_bakeMode.selection != 3 and superclassof BakePFlowToObjects_Proxy == Light and hasProperty BakePFlowToObjects_Proxy #multiplier then
								(
									theInstance.multiplier.controller = Bezier_Float()--set the multiplier controller to Bezier Float
									addMultiplierCurve theInstance.multiplier.controller --add a multiplier curve
									theMCurve = theInstance.multiplier.controller[2].controller
									theKey = addNewKey theMCurve t --set a key on the current frame
									theKey.value = 1 --set the on to 1 (enabled)
									theKey = addNewKey theMCurve (t-1) --set a key on the previous frame
									theKey.value = 0 --and set to disabled
								)
								else
								(
									theInstance.visibility = bezier_float() --set the visibility controller to Bezier Float
									theKey = addNewKey theInstance.visibility.controller t --set a visibility key on the current frame
									theKey.value = 1 --set the visibility to 1 (visible)
									theKey = addNewKey theInstance.visibility.controller (t-1) --set a vis. key on the previous frame
									theKey.value = 0 --and set to invisible
								)	
							)	
							InstancesArray[theID] = theInstance --store the instance in the array 
						)--end if no instance yet
						with animate  (ddl_timeMode.selection == 1) InstancesArray[theID].transform = BakePFlowToObjects_PFlow.particleTM --keyframe the TM of the current instance to match the particle
					)--end if ID > 0
				)--end p loop
				if ddl_timeMode.selection == 1 do --if doing multiframe baking, set visibility
				(
					for i in lastBornIDStorage where findItem bornIDStorage i == 0 do --for each BornID collected on the last frame that is NOT FOUND on this frame,
					(
						if ddl_bakeMode.selection != 3 and superclassof BakePFlowToObjects_Proxy == Light and hasProperty BakePFlowToObjects_Proxy #multiplier then
						(
							theInstance = InstancesArray[i]  --grab the instance object corresponding to the particle with that ID
							theKey = addNewKey theInstance.multiplier.controller[2].controller t --and set a key on this frame
							theKey.value = 0 --with a value of 0 (off)
							theKey = addNewKey theInstance.multiplier.controller[2].controller (t-1) --and another key on the previous frame
							theKey.value = 1 --with a value of 1 (on)
						)
						else
						(
							theInstance = InstancesArray[i]  --grab the instance object corresponding to the particle with that ID
							theKey = addNewKey theInstance.visibility.controller t --and set a visibility key on this frame
							theKey.value = 0 --with a value of 0 (invisible)
							theKey = addNewKey theInstance.visibility.controller (t-1) --and another key on the previous frame
							theKey.value = 1 --with a value of 1 (visible)
						)	
					)--end i loop
				)--end if	
			)--end t loop	
			progressEnd()
		)--end on
		
		on btn_deleteBaked pressed do try(delete (execute ("$'BAKE_" + BakePFlowToObjects_PFlow.name + "_instance'*")))catch() --delete objects based on the PFlow name
		on BakePFlowToObjects_Rollout open do updateEnabledState() --update the buttons when the dialog opens
		on BakePFlowToObjects_Rollout moved pos do setIniSetting theIniFile "Dialog" "Position" (pos as string) --store the current dialog position
	)--end rollout
	
	thePos = execute (getIniSetting theIniFile "Dialog" "Position") --get the dialog position from the ini file
	if thePos == OK do thePos = [100,100] --if it is not saved yet, assume a default position
	createDialog BakePFlowToObjects_Rollout 200 175 thePos.x thePos.y --open the dialog at the last position
)--end script